#!/usr/bin/env perl
use strict;
use warnings;
use 5.10.0;

use lib '/opt/linode/longview';

use JSON::PP;
use Sys::Hostname;
use LWP::UserAgent;
use Compress::Zlib;
use Linode::Longview::DataGetter qw(get);
use Linode::Longview::Util qw(constantPush flattenData slurp_file daemonize_self check_already_running $SLEEP_TIME);

use IO::Socket::INET6;
BEGIN {
    $Net::HTTP::SOCKET_CLASS = 'IO::Socket::INET6';
    require Net::HTTP;
}

die "LongView must be run as root in order to collect data\n" unless ($< == 0);
my $pid = check_already_running();
die "The LongView agent is already running as PID: $pid\n" if $pid;

my $ua = LWP::UserAgent->new(
	timeout => 10,
	agent   => "Linode Longview 1.0"
);
my $url = 'https://longview1.linode.com/cgi.pl';

my $confdir = "/etc/linode";
my $apiKeyFile = "$confdir/longview.key";

my $stats = {
	apikey  => scalar(slurp_file($apiKeyFile)),
	version => '1.0',
	payload => [],
};

unless ($stats->{apikey}){
	print "\nNo api key found. Please enter your API Key: ";
	chomp(my $key = <>);
	unless ($key =~ /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{16}\z$/){
		print "Invalid API Key\n";
		exit 1;
	}
	$stats->{apikey} = $key;
	umask 066;
	mkdir $confdir;
	open my $fh, '>', $apiKeyFile or die "Couldn't open $apiKeyFile for writing: $!";
	print $fh $key;
	close $fh;
}
die "Invalid API key\n" unless ($stats->{apikey} =~ /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{16}\z$/);

chown 0,0,$apiKeyFile;
chmod 0600,$apiKeyFile;

my ($debug, $quit) = (0,0);
daemonize_self() unless $debug;
$0 = "Linode-Longview";
$SIG{TERM} = $SIG{INT} = $SIG{HUP} = $SIG{QUIT} = sub { $quit = 1 };

my $data={};
my ($counter, $sleep) = (0, $SLEEP_TIME);
while (!$quit) {
	$data->{timestamp} = time;
	get($_,$data,) foreach (qw(Processes Uptime Ports Load Memory CPU Network Disk Packages SysInfo));

	constantPush($stats->{payload},$data);
	$data = {};

	my $req = $ua->post(
		$url,
		Content_Type => 'form-data',
		Content => [
			'data' => [
				undef, 
				'json.gz', 
				'Content-Type' => 'application/json',
				'Content-Encoding' => 'gzip',
				'Content' => Compress::Zlib::memGzip(encode_json($stats))
			]   
		]   
	);  

	$sleep = $SLEEP_TIME;
	if ($req->is_success){
		@{$stats->{payload}} = ();
		my $rep = decode_json($req->decoded_content);
		$sleep = $rep->{sleep} if defined $rep->{sleep};
		die "Server has requested this API Key stop sending data" if defined $rep->{die} && $rep->{die} eq "please";
	}
	if ($debug) {
		print $req->status_line . "\n";
		print $req->decoded_content . "\n" ;
		exit 0;
	}

	$counter++;
	sleep $sleep;
}
