#!/usr/bin/perl

package Linode::Longview::DataGetter::Processes;

use POSIX;
use Linode::Longview::Util qw(slurp_file delta $procfs);
use Linode::Longview::DataGetter::Uptime;

use Exporter 'import';
our @EXPORT_OK = (qw(get processInfo processList));

my $TICKS = POSIX::sysconf(&POSIX::_SC_CLK_TCK);

sub processList {
	my @procs = map { s|$procfs||; $_ } glob "${procfs}[0-9]*";
	return @procs;
}

sub processInfo {
	my $pid = shift;
	my %proc;

	# all or nothing, grab what we need asap
	my @procFile  = slurp_file( $procfs . "$pid/status" ) or return;
	my $stat_line = slurp_file( $procfs . "$pid/stat" )   or return;
	# some kernels don't have /proc/pid/io, let the for loop fall apart later
	my @iosample  = slurp_file( $procfs . "$pid/io" ) ;

	foreach my $line (@procFile) {
		if ( $line =~ m/^Name:\s+(.*)/ )         { $proc{name} = $1; }
		if ( $line =~ m/^Pid:\s+(.*)/ )          { $proc{pid}  = $1; }
		if ( $line =~ m/^PPid:\s+(.*)/ )         { $proc{ppid} = $1; }
		if ( $line =~ m/^Uid:\s+.*?\s+(.*?)\s/ ) { $proc{uid}  = $1; }
		if ( $line =~ m/^VmRSS:\s+(.*)\s+kB/ )   { $proc{mem}  = $1; }
		last if ( $line =~ m/^Threads/ );
	}

	$proc{user}  = ( getpwuid( $proc{uid} ) )[0];
	$proc{group} = ( getgrgid( $proc{gid} ) )[0];

	# cpu
	my ( $usertime, $systemtime, $startJiffies )
		= ( split /\s+/, $stat_line )[ 13, 14, 21 ];
	$proc{cpu} = $usertime + $systemtime;
	$proc{age} = procAge($startJiffies);

	# io
	foreach $line (@iosample) {
		$proc{ioreadkbytes}  = $1 if ( $line =~ m/^read_bytes:\s+(.*)/ );
		$proc{iowritekbytes} = $1 if ( $line =~ m/^write_bytes:\s+(.*)/ );
	}

	return %proc;
}

sub get {
	my (undef, $dataref) = @_;
	my @procs = processList();
	foreach my $proc (@procs) {
		my %info = processInfo($proc);
		next if ( !%info );    #No such proc, it probably died while we were gathering info
		next if ( $info{ppid} == 2 );
		next if ( $info{pid} == 2 );
		$info{name} =~ s/\./\\\./g;

		$dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.count"}++;
		$dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.mem"} 			+= $info{mem} + 0;
		$dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.cpu"} 			+= $info{cpu}			if (defined $info{cpu});
		$dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.ioreadkbytes"}	+= $info{ioreadkbytes}	if (defined $info{ioreadkbytes});
		$dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.iowritekbytes"}	+= $info{iowritekbytes}	if (defined $info{iowritekbytes});
		$dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.zz_age"} 			 = $info{age}
			if ((!defined($dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.zz_age"} )) || 
				$info{age} > $dataref->{LONGTERM}->{"Processes.${info{name}}.${info{user}}.zz_age"} );
	}

	foreach my $name (sort keys %{$dataref->{LONGTERM}}) {
		next unless $name =~ /^Processes/;
		if($name =~ /zz_age$/ ){
			delete $dataref->{LONGTERM}->{$name};
			next;
		}
		my $age_key = $name;
		$age_key =~ s/[^\.]*$/zz_age/;
		delete $dataref->{LONGTERM}->{$name} if ($dataref->{LONGTERM}->{$age_key} < 60);
	}

	return $dataref;
}

sub procAge {
	my $startJiffies   = shift;
	my $uptime = ( split( /\s+/, slurp_file( $procfs . 'uptime' ) ) )[0];
	my $currentJiffies = $uptime * $TICKS;
	my $deltaJiffies   = delta( $currentJiffies, $startJiffies );
	return $deltaJiffies / $TICKS;
}

1;
