package Linode::Longview::Util;

use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK
	= qw(slurp_file jiffies2Percent delta detect_system $procfs $sysfs $arch $SLEEP_TIME constantPush flattenData daemonize_self check_already_running);

use Carp;
use POSIX;

use feature 'state';

my $TICKS = POSIX::sysconf(&POSIX::_SC_CLK_TCK);
our $procfs = find_procfs()      or die "Couldn't find procfs: $!";
our $sysfs  = find_sysfs()       or die "Couldn't find sysfs: $!";
our $arch   = get_architecture() or die "Couldn't determine architecture: $!";
our $SLEEP_TIME = 60;
my $pid_file = "/var/run/longview.pid";

my $slots = 10;

# For an arbrary sized cache ($slots long), we want to remove an element to keep the average gap between
# any two elements as small as possible. Turns out that the right seqence to remove elements is a repeating sequence
# of runs of 1 .. $slots - 2, with ($slots -1) x ((2**runNumber) - 1) interleaved before each element of the run.
#
# The first push below handles the inserting ($slots -1) x ((2**runNumber) - 1), while the second adds current element of the run
# The loop is a while rather than a for, as the legth of the list becomes MUCH longer than the iteration currently being calculated,
# after just a few runs (~2), so it's easier to measure the length of the seqence directly than try to approximate it from the iteration number
sub removeSequence {
	my $iteration = shift;
	my $current = 0;
	my @sequence;
	while (scalar(@sequence) < $iteration) {
		# We never want to remove element 0, so candidates for removal are 1 through $slots - 1
		# $slots - 1 will be inserted before the beginning of each element starting in the second run,
		# so each run should cover the range 1 to $slots - 2. You can calculate which run you're on
		# by doing the integer division of $current by the top element of a run (ie $slots -2).
		push @sequence, $slots-1 for(1..(2**int($current/($slots-2)))-1);
		push @sequence,($current%($slots-2))+1;
		$current++;
	}
	@sequence = @sequence[0 .. $iteration-1];
	return @sequence if (wantarray);
	return $sequence[-1];
	
}

# A nice wrapper to memoize and hide the complexities of removeSequence when pushing on to a fixed sized cache
sub constantPush {
	my ($ar,$val) = @_;
	confess("Array Ref is undefined") unless (defined($ar));
	state %iteration;
	my $addr = substr $ar,6,9;
	$iteration{$addr} = 0 if(scalar(@$ar)==0);
	push @$ar,$val;
	# Remove sequence expects to start at 1, while the first removal doesn't need to happen until $slots
	# so we subtract ($slots -1) to keep the numbers lined up with what each side expects
	splice(@$ar,removeSequence($iteration{$addr}-($slots-1)),1) if (scalar(@$ar) > $slots);
	$iteration{$addr}++;
}

sub slurp_file {
	my $path = shift;
	open( my $fh, '<', $path ) or return;
	if (wantarray) {
		my @data = <$fh>;
		return @data;
	}
	my $data = join( '', <$fh> );
	chomp $data;
	return $data;
}

sub jiffies2Percent {
	my ( $old, $new, $wallSeconds ) = @_;
	my $wallJiffies = $wallSeconds * $TICKS;
	confess "div by 0, PANIC!" if ( $wallJiffies == 0 );
	my $delta = $new - $old;
	return ( $delta / $wallJiffies ) * 100;
}

sub detect_system {
	@cpu = slurp_file( $procfs . 'cpuinfo' );

	if ( -f $procfs . 'user_beancounters' ) {
		return 'openvz';
	}
	elsif (
		(   slurp_file(
				'/sys/devices/system/clocksource/clocksource0/current_clocksource'
			) eq 'kvm-clock'
		)
		|| (   ( grep {/QEMU Virtual CPU/} @cpu )
			&& ( grep {/hypervisor/} @cpu ) ) )
	{   
		return 'kvm';
	}
	elsif ( -e '/dev/vzfs' ) {
		return 'virtuozzo';
	}
	elsif ( -d $sysfs . 'bus/xen' ) {
		return 'xen';
	}
	return 'baremetal';
}

sub flattenData {
	my ($mlhr,$name) = @_;
	my $ret = {};
	foreach my $sk (keys %{$mlhr}) {
		if ((ref $mlhr->{$sk}) eq "HASH"){
			my $children = flattenData($mlhr->{$sk},$sk);
			foreach my $child (keys %{$children}) {
				my $k = $name?$name.".".$child:$child;
				$ret->{$k} = $children->{$child};
			}
		}
		else{
			my $k = $name?$name.".".$sk:$sk;
			$ret->{$k} = $mlhr->{$sk};
		}
	}
	return $ret;
}

sub delta {
	my ( $current, $old ) = @_;
	return "ROLLOVER" if ( $current < $old );
	return $current - $old;
}

sub get_architecture {
	my $arch = ( ( POSIX::uname() )[4] );
	return $arch;
}

sub find_procfs {
	return "/proc/" if -d "/proc/$$";

	my @mtab = grep {/\bproc\b/} ( slurp_file('/etc/mtab') )
		or die "Couldn't check /etc/mtab: $!";
	return ( split( /\s+/, $mtab[0] ) )[1] . '/';
}

sub find_sysfs {
	my @mtab = grep {/\bsysfs\b/} ( slurp_file('/etc/mtab') )
		or die "Couldn't check /etc/mtab: $!";
	return ( split( /\s+/, $mtab[0] ) )[1] . '/';
}

sub daemonize_self {
 	#<<<   perltidy ignore
 	chdir '/'                      or die("Can't chdir to /: $!");
 	open STDIN, '<', '/dev/null'   or die("Can't read /dev/null: $!");
 	open STDOUT, '>>', '/dev/null' or die("Can't write to /dev/null: $!");
 	open STDERR, '>>', '/dev/null' or die("Can't write to /dev/null: $!");
 	defined( my $pid = fork )      or die("Can't fork: $!");
 	exit if $pid;
 	setsid or die("Can't start a new session: $!");
 	umask 022;
 	system "echo $$ > $pid_file";
 	#>>>
}

sub check_already_running {
	return 0 unless (-e $pid_file);
	my $pid = slurp_file($pid_file);
	my $name = slurp_file("/proc/$pid/cmdline");
	return $pid if $name =~ /ongview/;
	return 0;
}

1;
